@@ -18,7 +18,7 @@ Part of this content comes from
 from typing import List
 import requests
 import time
-from os import device_encoding, path as osp
+from os import path as osp
 from os import makedirs
 from os import remove
 
@@ -39,24 +39,25 @@ except:
 from pyxtream.progress import progress
 
 class Channel():
-    stream_type = ""
-    logo_path = ""
-    logo = ""
-    info = ""
+    # Required by Hypnotix
     id = ""
+    name = "" # What is the difference between the below name and title?
+    logo = ""
+    logo_path = ""
+    group_title = ""
+    title = ""
     url = ""
 
-    # What is the difference between the below name and title?
-    name = ""
-    title = ""
-
-    # Group info
-    group_title = ""
+    # XTream
+    stream_type = ""
     group_id = ""
-
     is_adult = ""
     added = ""
     epg_channel_id = ""
+    added = ""
+    
+    # This contains the raw JSON data
+    raw = ""
 
     def __init__(self, xtream: object, group_title, stream_info):
         stream_type = stream_info['stream_type']
@@ -73,26 +74,38 @@ class Channel():
             self.raw = stream_info
 
             stream_name = stream_info['name']
+
+            # Required by Hypnotix
             self.id = stream_info['stream_id']
             self.name = stream_name
             self.logo = stream_info['stream_icon']
             self.logo_path = xtream.getLogoLocalPath(self.logo)
-
-            self.group_id = stream_info['category_id']
-
             self.group_title = group_title
             self.title = stream_name
 
+            # Check if category_id key is available
+            if "category_id" in stream_info.keys():
+                self.group_id = stream_info['category_id']
+
             if stream_type == "live":
                 stream_extension = "ts"
                 
-                self.is_adult = stream_info['is_adult']
-                self.epg_channel_id = stream_info['epg_channel_id']
+                # Default to 0
+                self.is_adult = 0
+                # Check if is_adult key is available
+                if "is_adult" in stream_info.keys():
+                    self.is_adult = stream_info['is_adult']
+
+                # Check if epg_channel_id key is available
+                if "epg_channel_id" in stream_info.keys():
+                    self.epg_channel_id = stream_info['epg_channel_id']
+
                 self.added = stream_info['added']
 
             elif stream_type == "movie":
                 stream_extension = stream_info['container_extension']
 
+            # Required by Hypnotix
             self.url = "{}/{}/{}/{}/{}.{}".format(
                 xtream.server,
                 stream_info['stream_type'],
@@ -116,10 +129,38 @@ class Channel():
         return jsondata
 
 class Group():
+    # Required by Hypnotix
+    name = ""
+    group_type = ""
+
+    # XTream
+    group_id = ""
+
+    # This contains the raw JSON data
+    raw = ""
+
+    def convert_region_shortname_to_fullname(self, shortname):
+
+        if (shortname == "AR"):
+            return "Arab"
+        elif (shortname == "AM"):
+            return "America"
+        elif (shortname == "AS"):
+            return "Asia"
+        elif (shortname == "AF"):
+            return "Africa"
+        elif (shortname == "EU"):
+            return "Europe"
+        else:
+            return ""
+
     def __init__(self, group_info: dict, stream_type: str):
         # Raw JSON Group
         self.raw = group_info
 
+        self.channels = []
+        self.series = []
+
         TV_GROUP, MOVIES_GROUP, SERIES_GROUP = range(3)
 
         if "VOD" == stream_type:
@@ -132,12 +173,30 @@ class Group():
             print("Unrecognized stream type `{}` for `{}`".format(
                 stream_type, group_info
             ))
+
         self.name = group_info['category_name']
-        self.group_id = group_info['category_id']
-        self.channels = []
-        self.series = []
+        split_name = self.name.split('|')
+        self.region_shortname = ""
+        self.region_longname = ""
+        if len(split_name) > 1:
+            self.region_shortname = split_name[0].strip()
+            self.region_longname = self.convert_region_shortname_to_fullname(self.region_shortname)
+
+        
+        # Check if category_id key is available
+        if "category_id" in group_info.keys():
+            self.group_id = group_info['category_id']
 
 class Episode():
+    # Required by Hypnotix
+    title = ""
+    name = ""
+
+    # XTream
+
+    # This contains the raw JSON data
+    raw = ""
+
     def __init__(self, xtream: object, series_info, group_title, episode_info) -> None:
         # Raw JSON Episode
         self.raw = episode_info
@@ -167,21 +226,47 @@ class Episode():
             print("{} - Bad URL? `{}`".format(self.name, self.url))
 
 class Serie():
+    # Required by Hypnotix
+    name = ""
+    logo = ""
+    logo_path = ""
+
+    # XTream
+    series_id = ""
+    plot = ""
+    youtube_trailer = ""
+    genre = ""
+
+    # This contains the raw JSON data
+    raw = ""
+
     def __init__(self, xtream: object, series_info):
         # Raw JSON Series
         self.raw = series_info
 
+        # Required by Hypnotix
         self.name = series_info['name']
-        self.series_id = series_info['series_id']
         self.logo = series_info['cover']
         self.logo_path = xtream.getLogoLocalPath(self.logo)
-        
-        self.seasons = {}
-        self.episodes = {}
 
-        self.plot = series_info['plot']
-        self.youtube_trailer = series_info['youtube_trailer']
-        self.genre = series_info['genre']
+        self.seasons = []
+        self.episodes = []
+
+        # Check if category_id key is available
+        if "series_id" in series_info.keys():
+            self.series_id = series_info['series_id']
+        
+        # Check if plot key is available
+        if "plot" in series_info.keys():
+            self.plot = series_info['plot']
+        
+        # Check if youtube_trailer key is available
+        if "youtube_trailer" in series_info.keys():
+            self.youtube_trailer = series_info['youtube_trailer']
+        
+        # Check if genre key is available
+        if "genre" in series_info.keys():
+            self.genre = series_info['genre']
 
     def export_json(self):
         jsondata = {}
@@ -192,6 +277,9 @@ class Serie():
         return jsondata
 
 class Season():
+    # Required by Hypnotix
+    name = ""
+
     def __init__(self, name):
         self.name = name
         self.episodes = {}
@@ -242,6 +330,12 @@ class XTream():
         self.password = provider_password
         self.name = provider_name
         self.cache_path = cache_path
+        
+        # get the pyxtream local path
+        self.app_fullpath = osp.dirname(osp.realpath(__file__))
+
+        # prepare location of local html template
+        self.html_template_folder = osp.join(self.app_fullpath,"html")
 
         # if the cache_path is specified, test that it is a directory
         if self.cache_path != "":
@@ -255,18 +349,20 @@ class XTream():
             self.cache_path = osp.expanduser("~/.xtream-cache/")
             if not osp.isdir(self.cache_path):
                 makedirs(self.cache_path, exist_ok=True)
+            print("pyxtream cache path located at {}".format(self.cache_path))
 
         self.authenticate()
 
         if USE_FLASK:
-            self.flaskapp = FlaskWrap('pyxtream', self)
+            self.flaskapp = FlaskWrap('pyxtream', self, self.html_template_folder)
             self.flaskapp.start()
 
-    def search_stream(self, keyword: str, return_type: str = "LIST") -> List:
+    def search_stream(self, keyword: str, ignore_case: bool = True, return_type: str = "LIST") -> List:
         """Search for streams
 
         Args:
             keyword (str): Keyword to search for. Supports REGEX
+            ignore_case (bool, optional): True to ignore case during search. Defaults to "True".
             return_type (str, optional): Output format, 'LIST' or 'JSON'. Defaults to "LIST".
 
         Returns:
@@ -275,7 +371,10 @@ class XTream():
 
         search_result = []
         
-        regex = re.compile(keyword)
+        if ignore_case:
+            regex = re.compile(keyword,re.IGNORECASE)
+        else:
+            regex = re.compile(keyword)
 
         print("Checking {} movies".format(len(self.movies)))
         for stream in self.movies:
@@ -322,7 +421,7 @@ class XTream():
 
         Args:
             url (str): Complete URL of the stream
-            fullpath_filename (str): Complete File path where to save the stream.
+            fullpath_filename (str): Complete File path where to save the stream
 
         Returns:
             bool: True if successful, False if error
@@ -429,12 +528,19 @@ class XTream():
 
     # Authentication returns information about the account and server:
     def authenticate(self):
-        r = requests.get(self.get_authenticate_URL())
-        self.authData = r.json()
-        self.authorization = {
-            "username": self.authData["user_info"]["username"],
-            "password": self.authData["user_info"]["password"]
+        self.authData = {}
+        try:
+            r = requests.get(
+                self.get_authenticate_URL()
+                )
+            self.authData = r.json()
+            self.authorization = {
+                "username": self.authData["user_info"]["username"],
+                "password": self.authData["user_info"]["password"]
             }
+        except requests.exceptions.ConnectionError:
+            # If connection refused
+            print("{} - Connection refused URL: {}".format(self.name, self.server))
 
     def loadFromFile(self, filename) -> dict:
         """Try to load the distionary from file
@@ -551,6 +657,7 @@ class XTream():
                     #provider.groups.append(new_group)
             else:
                 print("Could not load {} Groups".format(loading_stream_type))
+                break
 
             ## Get Streams
 
@@ -579,13 +686,26 @@ class XTream():
                 for stream_channel in all_streams:
                     # Generate Group Title
                     if stream_channel['name'][0].isalnum():
-                        group_title = str.split(stream_channel['name'],'|')[0]
 
                         # Some channels have no group, 
                         # so let's add them to the catche all group
                         if stream_channel['category_id'] == None:
                             stream_channel['category_id'] = '9999'
+                        elif stream_channel['category_id'] != '1':
+                            pass
+
+                        # Find the first occurence of the group that the
+                        # Channel or Stream is pointing to
+                        the_group = next(
+                            (x for x in self.groups if x.group_id == stream_channel['category_id']),
+                            None
+                        )
                         
+                        if the_group != None:
+                            group_title = the_group.name
+                        else:
+                            group_title = self.catch_all_group.name
+
                         if loading_stream_type == self.seriesType:
                             # Load all Series
                             new_series = Serie(self, stream_channel)
@@ -602,35 +722,22 @@ class XTream():
                                 stream_channel
                             )
 
-                        # Find the first occurence of the group that the 
-                        # Channel or Stream is pointing to
-                        the_group = next(
-                            (x for x in self.groups if x.group_id == stream_channel['category_id']),
-                            None
-                        )
-
-                        # Save the new channel to the provider object and the new_group object
+                        # Save the new channel to the local list of channels
                         if loading_stream_type == self.liveType:
                             self.channels.append(new_channel)
-                            #provider.channels.append(new_channel)
                         elif loading_stream_type == self.vodType:
                             self.movies.append(new_channel)
-                            #provider.movies.append(new_channel)
                         else:
                             self.series.append(new_series)
-                            #provider.series.append(new_series)
-                        
-                        if loading_stream_type != self.seriesType:
-                            #self.channels.append(new_channel)
-                            if the_group != None:
+
+                        # Add stream to the specific Group       
+                        if the_group != None:
+                            if loading_stream_type != self.seriesType:
                                 the_group.channels.append(new_channel)
                             else:
-                                print("Group not found `{}`".format(stream_channel['name']))
-                        else:
-                            if the_group != None:
                                 the_group.series.append(new_series)
-                            else:
-                                print("Group not found `{}`".format(stream_channel['name']))
+                        else:
+                            print("Group not found `{}`".format(stream_channel['name']))
             else:
                 print("Could not load {} Streams".format(loading_stream_type))
 
